#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#define F_CPU 12000000UL
#include "util/delay.h"
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"


#define LEDR_ON PORTB|=_BV(PB3)
#define LEDR_OFF PORTB&=~(_BV(PB3))
#define LEDG_ON PORTB|=_BV(PB2)
#define LEDG_OFF PORTB&=~(_BV(PB2))
#define LEDB_ON PORTB|=_BV(PB1)
#define LEDB_OFF PORTB&=~(_BV(PB1))

static uchar    reportBuffer[5];    /* buffer for HID reports, type game cntrler*/
static uchar    idleRate, posuv=5;        /* in 4 ms units */

static uchar    currentPosition;
static uchar    bytesRemaining;
static uchar 	prijata_data[100]={0,1,0,0,1};
static uchar	odeslana_data[3] = {55,42,8};
static uchar	prijata=0;

PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH]={/* USB report descriptor */
 	0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop) 
    0x09, 0x01,                    // USAGE (Vendor Usage 1) 
    0xa1, 0x01,                    // COLLECTION (Application) 
    0x19, 0x01,                    //   USAGE_MINIMUM (0) 
    0x29, 0x03,                    //   USAGE_MAXIMUM (3) 
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0) 
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255) 
    0x75, 0x08,                    //   REPORT_SIZE (8) 
    0x95, 0x03,                    //   REPORT_COUNT (3) 
    0x81, 0x02,                    //   INPUT (Data,Var,Abs) 
	
    0x19, 0x01,                    //   USAGE_MINIMUM (0) 
    0x29, 0x03,                    //   USAGE_MAXIMUM (3) 
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0) 
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8) 
    0x95, 0x03,                    //   REPORT_COUNT (3)
	0x91, 0x02,                    //   OUTPUT (Data,Var,Abs) 
    0xc0                           // END_COLLECTION

};

/* ------------------------------------------------------------------------- */

static void buildReport(int pozice)
{

	odeslana_data[0] = prijata_data[pozice];
	odeslana_data[1] = prijata_data[pozice+1];
	odeslana_data[2] = prijata_data[pozice+2];
	
}
//-------------------------------------------------------------------------

ISR(TIMER0_OVF_vect){

	wdt_reset();
	usbPoll();

}

//----------------------------------------------------------------
uchar   usbFunctionWrite(uchar *data, uchar len)
{
 	uchar i;
    if(len > bytesRemaining)                // if this is the last incomplete chunk
    	len = bytesRemaining;               // limit to the amount we can store
    bytesRemaining -= len;
    for(i = 0; i < len; i++)
        prijata_data[currentPosition++] = data[i];
	prijata=1;
    return bytesRemaining == 0; 
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar setupData[8])
{
   usbRequest_t *rq = (void *)setupData;   			// cast to structured data for parsing
   if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){ 	
    if(rq->bRequest == USBRQ_HID_SET_REPORT){
    	    currentPosition = 0;  						// initialize position index
			bytesRemaining=3;              		
    	    /*bytesRemaining = rq->wLength.word;  		// store the amount of data requested
    	    if(bytesRemaining > sizeof(prijata_data)) 	// limit to buffer size
     	       bytesRemaining = sizeof(prijata_data);*/
       		return USB_NO_MSG;        					// tell driver to use usbFunctionWrite()
    }
   }
    return 0;                               			// ignore all unknown requests
}

/* ------------------------------------------------------------------------- */
//-------- MAIN -----------------------------------------
//--------------------------------------------------------
int main() {
    uchar i,a,b;
	DDRB|=_BV(PB1)|_BV(PB2)|_BV(PB3);			//LED
	DDRB&=~(_BV(PB0));		//tl
	PORTB|=_BV(PB0); 		//tl
	TCCR0 = 0b00000101;		//citac 0, preddelicka 64
	TIMSK = 0b00000001;		//preruseni - preteceni
	TCCR2|=_BV(WGM20)|_BV(WGM21)|_BV(COM21)|_BV(CS22)|_BV(CS20);
	TCCR1A|=_BV(COM1A1)|_BV(COM1B1)|_BV(WGM10);
	TCCR1B|=_BV(WGM12)|_BV(CS11);

	    
	wdt_enable(WDTO_1S); // enable 1s watchdog timer
    
	odDebugInit();
	DBG1(0x00, 0, 0);       /* debug output: main starts */
    usbInit();

	usbDeviceDisconnect(); // enforce re-enumeration

    for(i = 0; i<250; i++) { // wait 500 ms
        wdt_reset(); // keep the watchdog happy
        _delay_ms(2);
    }
	a=1;
	b=0;
    usbDeviceConnect();

    sei(); // Enable interrupts after re-enumeration
	
	DBG1(0x01, 0, 0);       /* debug output: main loop starts */
    while(1) {
        DBG1(0x02, 0, 0);   /* debug output: main loop iterates */

		if(bit_is_clear(PINB,PB0)){
			

			if(OCR1B>255)
				OCR1B=0;
			if(usbInterruptIsReady()){ /* posledn peruen u je vyzeno - meme poslat */
           	 	buildReport(2);
            	usbSetInterrupt(odeslana_data, 3);
				//i++;
        		}
			//OCR2++;
			//OCR1B++;
			_delay_ms(10);
			}
		//--------
		
		odeslana_data[0] = 1;
		for(a=0;a<250;a=a+4){
				odeslana_data[1] = a;
				if(usbInterruptIsReady())
            		usbSetInterrupt(odeslana_data, 3);
				_delay_ms(10);
				if(prijata>0){
					prijata=0;
					OCR2=a;
					//a=255;
					}
		}
		
		odeslana_data[0] = 2;
		for(a=0;a<250;a=a+4){
				odeslana_data[1] = a;
				if(usbInterruptIsReady())
            		usbSetInterrupt(odeslana_data, 3);
				_delay_ms(10);
				if(prijata>0){
					prijata=0;
					OCR1B=a;
					}
		}
		odeslana_data[0] = 3;
		for(a=0;a<250;a=a+4){
				odeslana_data[1] = a;
				if(usbInterruptIsReady())
            		usbSetInterrupt(odeslana_data, 3);
				_delay_ms(10);
				if(prijata>0){
					prijata=0;
					OCR1A=a;
					}
		}
			
			
		
		

    }//while

    return 0;
}
