#include<avr/io.h>
#include<stdio.h> 
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#define F_CPU 12000000UL
#include "util/delay.h"

#define LED_ON PORTB|=_BV(PB1)
#define LED_OFF PORTB&=~(_BV(PB1))

static uchar    reportBuffer[5];    /* buffer for HID reports, type game cntrler*/
static uchar    idleRate, posuv=5;        /* in 4 ms units */



PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { /* USB report descriptor */
    0x05, 0x01,        // USAGE_PAGE (Generic Desktop = 01)
	0x09, 0x05,        // USAGE (Game Pad = 05)
	0xa1, 0x01,        // COLLECTION (Application)
	0x09, 0x01,        //   USAGE (Pointer)
	0xa1, 0x00,        //   COLLECTION (Physical)
	0x09, 0x30,        //     USAGE (X)
	0x09, 0x31,        //     USAGE (Y)
	0x15, 0x00,        //   LOGICAL_MINIMUM (0)
	0x26, 0xff, 0x00,  //     LOGICAL_MAXIMUM (255)
	0x75, 0x08,        //   REPORT_SIZE (8bits)
	0x95, 0x04,        //   REPORT_COUNT (2)
	0x81, 0x02,        //   INPUT (Data,Var,Abs)
	0xc0,              // END_COLLECTION
	
	0x05, 0x09,        // USAGE_PAGE (Button)
	0x19, 0x01,        //   USAGE_MINIMUM (Button 1)
	0x29, 0x08,        //   USAGE_MAXIMUM (Button 8)
	0x15, 0x00,        //   LOGICAL_MINIMUM (0)
	0x25, 0x01,        //   LOGICAL_MAXIMUM (1)
	0x75, 0x01,        // REPORT_SIZE (1bit)
	0x95, 0x08,        // REPORT_COUNT (8)
	0x81, 0x02,        // INPUT (Data,Var,Abs)
	0xc0               // END_COLLECTION
};

/* ------------------------------------------------------------------------- */

static void buildReport(void)
{

	reportBuffer[0] = posuv;
	reportBuffer[1] = posuv+1;
	reportBuffer[2] = posuv+4;
	reportBuffer[3] = posuv+3;
	reportBuffer[4] = 0x01; //endpoint1 ?

}
//-------------------------------------------------------------------------


uchar	usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    usbMsgPtr = reportBuffer;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
												   /* we only have one report type, so don't look at wValue */
            buildReport();
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
	return 0;
}



//-------- MAIN -----------------------------------------
//--------------------------------------------------------
int main() {
        uchar i;
	DDRB|=_BV(PB1);			//LED
	DDRB&=~(_BV(PB0));		//tl
	PORTB|=_BV(PB0); 		//tl

	
    
	wdt_enable(WDTO_1S); // enable 1s watchdog timer
    usbInit();
	odDebugInit();
    usbDeviceDisconnect(); // enforce re-enumeration

    for(i = 0; i<250; i++) { // wait 500 ms
        wdt_reset(); // keep the watchdog happy
        _delay_ms(2);
    }
    usbDeviceConnect();

    sei(); // Enable interrupts after re-enumeration
	

    while(1) {
        wdt_reset(); // keep the watchdog happy
        usbPoll();
		if(bit_is_clear(PINB,PB0)){
			posuv++;
			LED_OFF;
			}
		else
			LED_ON;

		if(usbInterruptIsReady() ){ /* posledn peruen u je vyzeno - meme poslat */
            buildReport();
            usbSetInterrupt(reportBuffer, sizeof(reportBuffer));

        }
    }

    return 0;
}
