/* Name: main.c
 * Project: hid-data, example how to use HID for data transfer
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-11
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 777 2010-01-15 18:34:48Z cs $
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/eeprom.h>

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[38] = {    /* USB report descriptor */
	0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop) 
    0x09, 0x01,                    // USAGE (Vendor Usage 1) 
    0xa1, 0x01,                    // COLLECTION (Application) 
    0x19, 0x01,                    //   USAGE_MINIMUM (0) 
    0x29, 0x03,                    //   USAGE_MAXIMUM (3) 
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0) 
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255) 
    0x75, 0x08,                    //   REPORT_SIZE (8) 
    0x95, 0x03,                    //   REPORT_COUNT (3) 
    0x81, 0x02,                    //   INPUT (Data,Var,Abs) 
    0x19, 0x01,                    //   USAGE_MINIMUM (0) 
    0x29, 0x03,                    //   USAGE_MAXIMUM (3) 
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0) 
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8) 
    0x95, 0x03,                    //   REPORT_COUNT (3)
    0x91, 0x02,                    //   OUTPUT (Data,Var,Abs) 
    0xc0                           // END_COLLECTION
};
/* Since we define only one feature report, we don't use report-IDs (which
 * would be the first byte of the report). The entire report consists of 128
 * opaque data bytes.
 */

/* The following variables store the status of the current data transfer */
static uchar    currentPosition;
static uchar    bytesRemaining;
static uchar 	prijata_data[20];
static uchar	odeslana_data[3] = {55,42,8};

/* ------------------------------------------------------------------------- */

ISR(TIMER0_OVF_vect){

	wdt_reset();
	usbPoll();

}

/* ------------------------------------------------------------------------- */

/* usbFunctionWrite() is called when the host sends a chunk of data to the
 * device. For more information see the documentation in usbdrv/usbdrv.h.
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
 	uchar i;
    if(len > bytesRemaining)                // if this is the last incomplete chunk
    	len = bytesRemaining;               // limit to the amount we can store
    bytesRemaining -= len;
    for(i = 0; i < len; i++)
        prijata_data[currentPosition++] = data[i];
    return bytesRemaining == 0;             // return 1 if we have all data
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar setupData[8])
{
    usbRequest_t *rq = (void *)setupData;   			// cast to structured data for parsing
    switch(rq->bRequest){
		case USBRQ_HID_SET_REPORT:
    	    currentPosition = 0;                		// initialize position index
    	    bytesRemaining = rq->wLength.word;  		// store the amount of data requested
    	    if(bytesRemaining > sizeof(prijata_data)) 	// limit to buffer size
     	       bytesRemaining = sizeof(prijata_data);
       		return USB_NO_MSG;        					// tell driver to use usbFunctionWrite()
    }
    return 0;                               			// ignore all unknown requests
}

/* ------------------------------------------------------------------------- */

int main(void)
{
TCCR0 = 0b00000101;		//citac 0, preddelicka 64
TIMSK = 0b00000001;		//preruseni - preteceni

DDRB = 0xff;
PORTB = 0b00100000;
uchar   i;

    wdt_enable(WDTO_1S);
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
    odDebugInit();
    DBG1(0x00, 0, 0);       /* debug output: main starts */
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();
    DBG1(0x01, 0, 0);       /* debug output: main loop starts */
    for(;;){                /* main event loop */
        DBG1(0x02, 0, 0);   /* debug output: main loop iterates */

		if(prijata_data[0] == 15)	PORTB |= 0b00010000;
		else	PORTB &= 0b11101111;
		if(prijata_data[1] == 77)	PORTB &= 0b11011111;
		else	PORTB |= 0b00100000;

		_delay_ms(200);

		if(usbInterruptIsReady()){              // only if previous data was sent
    		uchar *p = odeslana_data;
			uchar len = sizeof(odeslana_data);  // obtain chunk of max 8 bytes
    		if(len > 0)                         // only send if we have data
        		usbSetInterrupt(p, len);
		}
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
